@@ -13,6 +13,15 @@
 #include "inc_stl.h"
 
 
+
+enum eListIndexType
+{
+    LINDEX_NUMBERS,
+    LINDEX_VECTOR
+};
+
+
+
 //=============================================================================
 
 int CList::Write(FILE *f)
@@ -170,6 +179,9 @@ public:
 	ListElemFunction(const char *n) : Function(n,2,tlist,tnumber) {};
 	virtual Value Execute(int arity,Value *arg);
 	virtual int ValidArguments(int arity,Value *arg);
+
+private:
+	eListIndexType indexType_;
 };
 
 int ListElemFunction::ValidArguments(int arity,Value *arg)
@@ -177,44 +189,86 @@ int ListElemFunction::ValidArguments(int arity,Value *arg)
 	if(arity<2 || arity > 4) return false;
 	if(arg[0].GetType() != tlist) return false;
 
-	for(int i=1;i<arity;i++)
-		if(arg[i].GetType() != tnumber)
+
+	// indexing can either be a set of numbers or a single vector
+
+	if (arg[1].GetType() == tvector)  // vector
+	{
+		if (arity > 2)
 			return false;
+		else
+		{
+			indexType_ = LINDEX_VECTOR;
+			return true;
+		}
+	}
+	else  // number(s)
+	{
+		indexType_ = LINDEX_NUMBERS;
+
+		for(int i=1;i<arity;i++)
+			if(arg[i].GetType() != tnumber)
+				return false;
+	}
 
 	return true;
 }
 
 Value ListElemFunction::Execute(int arity,Value *arg)
 {
-	CList *v;
-	int  n;
-	int  m = 0;
-	int  s = 0;
+	if (indexType_ == LINDEX_NUMBERS)
+	{
+		CList *v;
+		int  n;
+		int  m = 0;
+		int  s = 0;
 
-	arg[0].GetValue(v);
-	arg[1].GetValue(n);
-	if(arity>2) arg[2].GetValue(m);
-	if(arity>3) arg[3].GetValue(s);
+		arg[0].GetValue(v);
+		arg[1].GetValue(n);
+		if(arity>2) arg[2].GetValue(m);
+		if(arity>3) arg[3].GetValue(s);
 
-	if(m <  n) m = n;
-	if(m >  v->Count()) m = v->Count();
-	if(s <= 0) s = 1;
+		if(m <  n) m = n;
+		if(m >  v->Count()) m = v->Count();
+		if(s <= 0) s = 1;
 
-	if(n <1 || n>v->Count())
-		return Error("Index out of range. Index is %d, list size is %d",
-			n,v->Count());
+		if(n <1 || n>v->Count())
+			return Error("Index out of range. Index is %d, list size is %d",
+				n,v->Count());
 
-	if(m == n ) return (*v)[n-1];
+		if(m == n ) return (*v)[n-1];
 
 
-	int c = (m-n)/s+1;
-	CList *l = new CList(c);
-	int i,j;
+		int c = (m-n)/s+1;
+		CList *l = new CList(c);
+		int i,j;
 
-	for(i = 0, j = n-1 ; i < c ; i++, j += s)
-		(*l)[i] = (*v)[j];
+		for(i = 0, j = n-1 ; i < c ; i++, j += s)
+			(*l)[i] = (*v)[j];
 
-	return Value(l);
+		return Value(l);
+	}
+
+	else  // vector indexing (i.e. the index is itself a vector of indexes)
+	{
+		CList   *v;
+		CVector *vi;
+		arg[0].GetValue(v);
+		arg[1].GetValue(vi);
+
+		// the result vector
+		CList *vnew = new CList(vi->Count());
+		for (int i = 0; i < vi->Count(); i++)
+		{
+			int index = vi->getIndexedValue(i);
+			if (index < 1 || index > v->Count())
+				return Error("index %d(%d) is out of range. List size is %d", i+1, index, v->Count());
+
+			(*vnew)[i] = (*v)[index-1];  // -1 to convert from 1-index (Macro) to 0-index (C)
+		}
+
+		return Value(vnew);
+	}
 }
 
 //=============================================================================
@@ -322,7 +376,7 @@ Value ListFindFunction::Execute(int arity,Value *arg)
         if (s && !strcmp(s, "all"))
             findAll = true;
         else
-            marslog(LOG_WARN, "find(any, list, string) only accepts 'all' as its last argument. Supplied: '%s'", s);
+            marslog(LOG_WARN, "find(list, any, string) only accepts 'all' as its last argument. Supplied: '%s'", s);
     }
 
     arg[0].GetValue(v);
