@@ -13,6 +13,8 @@
 #include "macro.h"
 #include "script.h"
 #include "value.h"
+
+#include "MvPath.hpp"
 #include "MvRequest.h"
 #include "MvApplication.h"
 
@@ -150,7 +152,7 @@ void CRequest::Print(void)
 	}
 }
 
-static void SetValue(request *r,const char *param,Value &val)
+void SetValue(request *r,const char *param,Value &val)
 {
 
 	double sd;
@@ -212,6 +214,10 @@ static void SetValue(request *r,const char *param,Value &val)
 						add_value(r,param,"%s",buf);
 						break;
 
+               case tnil:
+                  add_value(r,param,"%.12g",DBL_MAX);
+                  break;
+
 					default:
 						x[j].GetValue(p);
 						if( (p = clone_all_requests(p)) )
@@ -611,8 +617,10 @@ Value SimpleRequestFunction::Execute(int arity ,Value *arg)
 		set_subrequest(r,"_DEVICE_DRIVER",s);  //-- directrly for new PlotMod
 	   }
 #endif
-	   set_value(r,"_MACRO","%s",Script::MacroPath());
-	   set_value(r,"_PATH","%s",Script::MacroMainPath());
+
+      set_value(r,"_MACRO","%s",Script::MacroPath());
+      string fullPath = MakeAbsolutePath((const char*)get_value(r,"PATH",0),mdirname(Script::MacroMainPath()));
+      set_value(r,"_PATH",fullPath.c_str());
 	}
 
 	v = service ? Value(service,r,attachedContent) : Value(r);
@@ -1007,7 +1015,8 @@ Value ComplexRequestFunction::Execute(int arity ,Value *arg)
     }
 
     set_value(p,"_MACRO","%s",Script::MacroPath());
-    set_value(p,"_PATH","%s",Script::MacroMainPath());
+    string fullPath = MakeAbsolutePath((const char*)get_value(p,"PATH",0),mdirname(Script::MacroMainPath()));
+    set_value(p,"_PATH",fullPath.c_str());
 
     Value v(Service(),p);
     free_all_requests(p);
@@ -1104,9 +1113,11 @@ Value PlotFunction::ExecuteComplexRequestFunction(int arity ,Value *arg)
 		}
 	}
 
-	set_value(p,"_MACRO","%s",Script::MacroPath());
-	set_value(p,"_PATH","%s",Script::MacroMainPath());
-	set_value(p,"_NAME",Context::UniqueName());
+   // Add hidden parameters
+   set_value(p,"_MACRO","%s",Script::MacroPath());
+   set_value(p,"_NAME",Context::UniqueName());
+   string fullPath = MakeAbsolutePath((const char*)get_value(p,"PATH",0),mdirname(Script::MacroMainPath()));
+   set_value(p,"_PATH",fullPath.c_str());
 
 	// Define output requests according to the following rules/priorities:
 	// 1. Drivers defined in the "plot" command have top priority (overwrite "setoutput")
@@ -1153,7 +1164,8 @@ class PageFunction : public ComplexRequestFunction{
     virtual Value Execute(int arity,Value *arg);
 	virtual char* Service() { return PlotterFunction::Plotter(); }
 public:
-	PageFunction(const char *n) : ComplexRequestFunction(n,"VisModTrans") {};
+	PageFunction(const char *n) : ComplexRequestFunction(n,"VisModTrans")
+	    {info="Forces a new page on PostScript output";};
 };
 
 Value PageFunction::Execute(int ,Value*)
@@ -1367,7 +1379,8 @@ void CRequest::SetSubValue(Value& v,int arity,Value *arg)
 
 class PlotSuperpageFunction : public Function {
 public:
-	PlotSuperpageFunction(const char *n) : Function(n,2,tstring,trequest) {};
+	PlotSuperpageFunction(const char *n) : Function(n,2,tstring,trequest)
+	    {info="Returns a super_page for plotting";};
 	virtual Value Execute(int arity,Value *arg);
 	virtual int ValidArguments(int arity,Value *arg);
 	request *GetRequest(int arity ,Value *arg);
