@@ -47,6 +47,7 @@ MapView::MapView( Page&            owner,
 	PlotModView ( owner, viewRequest, setupRequest ),
 	PlotModService   ( this )
 {
+#if 0
 	// Create a coastline request, if one does not exist
 	MvRequest coastRequest = viewRequest_.getSubrequest ("COASTLINES");
 	if ( !ObjectList::IsVisDefCoastlines(coastRequest.getVerb()) )
@@ -54,6 +55,7 @@ MapView::MapView( Page&            owner,
 		coastRequest = ObjectList::UserDefaultRequest ( "MCOAST" );
 		viewRequest_ ( "COASTLINES" ) = coastRequest;
 	}
+#endif
 }	
 
 MapView::MapView(const MapView &old) :
@@ -120,7 +122,7 @@ MapView::Drop ( PmContext& context )
          this->UpdateView ( request );
 
       else if ( ObjectList::IsVisDefCoastlines ( verb ) )
-         this->ProcessCoastlines ( request, foundDU );
+         Owner().ProcessCoastlines ( request, PlotMod::Instance().CalledFromMacro(), foundDU );
 
       else if ( ObjectList::IsService ( verb,"visualise",true ) )
       {
@@ -152,7 +154,6 @@ MapView::Drop ( PmContext& context )
          // Redraw this page
          if (  request ("_APPL") != Cached ("macro") )
          {
-            Owner().EraseDraw();
             Owner().RedrawIfWindow();
          }
       }
@@ -216,7 +217,7 @@ void MapView::DecodeDataUnit ( MvIcon& dataUnit )
     // Read data headers (one by one)
     // retrieve the data offset and matching info
     // Pass this information to the page for matching
-    int dimFlag = 0;
+//    int dimFlag = 0;  //FAMI20170210 maybe there is not need for this flag
     bool swap;
     while ( decoder->ReadNextData() )
     {
@@ -238,7 +239,7 @@ void MapView::DecodeDataUnit ( MvIcon& dataUnit )
                     !matchingCriteria_.Match ( dataInfo, nextDataInfo ))
                 {
                     // No, so plot them separately
-                    dimFlag = dimFlag | 1;
+//                    dimFlag = dimFlag | 1;
                     if ( Owner().InsertDataUnit ( dataUnit.Id(), offset, offset, dataInfo, subpageId) == false ) break;
 
                     if ( Owner().InsertDataUnit ( dataUnit.Id(), nextDataOffset, nextDataOffset, nextDataInfo, subpageId ) == false ) break;
@@ -248,7 +249,7 @@ void MapView::DecodeDataUnit ( MvIcon& dataUnit )
                     // Yes, so plot them together.
                     // Insert dataunit using the correct order of the offsets
                     // (i.e. u/v not v/u). This is a Magics requirement.
-                    dimFlag = dimFlag | 2;
+//                    dimFlag = dimFlag | 2;
                     bool flag;
                     if ( swap )
                        flag = Owner().InsertDataUnit ( dataUnit.Id(), nextDataOffset, offset, dataInfo, subpageId );
@@ -261,14 +262,14 @@ void MapView::DecodeDataUnit ( MvIcon& dataUnit )
             }
             else // An isolated field, who just happens to have a companion defined.
             {
-                dimFlag = dimFlag | 1;
+//                dimFlag = dimFlag | 1;
                 if ( Owner().InsertDataUnit ( dataUnit.Id(), offset, nextDataOffset, dataInfo, subpageId) == false ) break;
             }
         }
         else
         {
             // It is an isolated field
-            dimFlag = dimFlag | 1;
+//            dimFlag = dimFlag | 1;
             if ( Owner().InsertDataUnit ( dataUnit.Id(), offset, nextDataOffset, dataInfo, subpageId) == false )
                 break;
         }
@@ -305,24 +306,19 @@ void MapView::DecodeDataUnit ( MvIcon& dataUnit )
         empty = false;
     }
 
-    // Some decoders will want to update the data requests
+    // Some decoders need to update the data request
     MvRequest dataRequest;
     if ( decoder->UpdateDataRequest(dataRequest) )
-        dataBase.UpdateIcon(DB_DATAUNIT,dataUnit.Id(),dataRequest);
-
-     // If the dataset contains single and companion fields then
-     // tell Magics to plot single and companion fields separately
-     // Otherwise, if there is no Overlay Control customized by the user
-     // then always overlay
-//     if ( single && companion )
-//          viewRequest_("MAP_OVERLAY_CONTROL") = "NEVER";
-//     else if ( !viewRequest_.getSubrequest("OVERLAY_CONTROL") )
-//          viewRequest_("SUBPAGE_OVERLAY_MODE") = "ALWAYS";
+    {
+       // update database and icon structure
+       dataBase.UpdateIcon(DB_DATAUNIT,dataUnit.Id(),dataRequest);
+       dataUnit.SaveRequest(dataRequest);
+    }
 
     // Update icon request to indicate a vector drawing
-    MvRequest req = dataUnit.Request();
-    req("_NDIM_FLAG") = dimFlag;
-    dataUnit.SaveRequest(req);
+//    MvRequest req = dataUnit.Request();
+//    dataRequest("_NDIM_FLAG") = dimFlag;
+//    dataUnit.SaveRequest(dataRequest);
 }
 
 void
@@ -346,21 +342,24 @@ MapView::ConvertToSatellite ( MvRequest& decRequest )
 }
 
 void
-MapView::UpdateView ( MvRequest& viewRequest )
+MapView::UpdateView ( MvRequest& viewReq )
 {
    // uPlot recognises that MapView and GeoView are basically the same thing
-   if ( ObjectList::IsGeographicalView( viewRequest.getVerb() ) )
-//   if ( strcmp (viewRequest_.getVerb(),viewRequest.getVerb()) == 0)
+   if ( ObjectList::IsGeographicalView( viewReq.getVerb() ) )
    {
       // Ensure coastlines exist
-      viewRequest_ = viewRequest;
-      MvRequest coastRequest = viewRequest_.getSubrequest ("COASTLINES");
+      MvRequest coastRequest = viewReq.getSubrequest ("COASTLINES");
       if ( ! coastRequest )
       {
          coastRequest = ObjectList::UserDefaultRequest ( "MCOAST" );
-         viewRequest_ ( "COASTLINES" ) = coastRequest;
+         viewReq ( "COASTLINES" ) = coastRequest;
       }
 
+      // Update the Owner View. The Owner will also update the local viewRequest_
+      // structure (the Owner has a pointer to it). There is no need to have
+      // command: viewRequest_ = viewReq;
+      Owner().UpdateView(viewReq);
+
       // Redraw this page
       Owner().RedrawIfWindow();
       Owner().NotifyObservers();
@@ -371,46 +370,21 @@ MapView::UpdateView ( MvRequest& viewRequest )
    {
       // Changing View is disabled at the moment
       char text[128];
-      sprintf(text,"Changing View (%s to %s) is currently disabled",viewRequest_.getVerb(),viewRequest.getVerb() );
+      sprintf(text,"Changing View (%s to %s) is currently disabled",viewRequest_.getVerb(),viewReq.getVerb() );
       PlotMod::Instance().errorMessage(text);
 
       // The code below needs to be updated if uPlot will allow changing views
       #if 0
       RecreateData();
-      viewRequest_ = viewRequest;
+      viewRequest_ = viewReq;
 
       // Create a view associated to the page
-      auto_ptr <PlotModView> view ( PlotModViewFactory::Make (Owner(), viewRequest ) );
+      auto_ptr <PlotModView> view ( PlotModViewFactory::Make (Owner(), viewReq ) );
       Owner().SetView(view);
       #endif
    }
 }
 
-void
-MapView::ProcessCoastlines ( MvRequest& coastRequest, bool foundDU )
-{
-    // Two main procedures:
-    // A) if the icon came from a Macro then keep it in the same position
-    //    as defined in the Plot command
-    // B) if the icon came from a Drop then replace the one defined in
-    //    the View request
-
-    // Icon from a Macro
-    if ( PlotMod::Instance().CalledFromMacro() )
-    {
-        MvIconDataBase& dataBase = Owner().IconDataBase();
-        coastRequest("_PLOTTING_ORDER") = foundDU ? "FOREGROUND" : "BACKGROUND";
-        MvIcon icon(coastRequest,true);
-        dataBase.InsertIcon( PRES_VISDEF_REL, Owner().Id(), icon, -1, false );
-    }
-    else // icon from a Drop
-        viewRequest_ ( "COASTLINES" ) = coastRequest;
-
-    // Redraw this page
-    Owner().EraseDraw();
-    Owner().RedrawIfWindow();
-}
-
 #if 0
 void
 MapView::ReplaceArea ( const Location& coordinates, int izoom)
@@ -438,7 +412,6 @@ MapView::ReplaceArea ( const Location& coordinates, int izoom)
 void
 MapView::DrawBackground ( )
 {
-#if 1   // New code
    // Retrieve background request
    MvRequest backList;
    if ( this->RetrieveBackground(backList) == 0 )
@@ -450,91 +423,19 @@ MapView::DrawBackground ( )
    {
       // Draw the layer info
       MvRequest req = backList.justOneRequest();
-      MvIcon icon(req,true);
-      Owner().DrawLayerInfo( icon.Id() );
+      string icon_name = (const char*)req("_NAME") ? mbasename((const char*)req("_NAME")): "<coastlines>";
+      Owner().DrawLayerInfo( (int)req("_ID"), icon_name.c_str() );
 
       // Ask the graphics engine to draw the coastlines
       ge.Draw ( req, true );
    
       backList.advance();
    }
-
-   Owner().SetProjection ( viewRequest_ );
-
-#else   // Old code, delete it later
-   bool draw = false;
-   string ON = "ON";
-
-   // Get DrawingPriority from Page
-   DrawingPriority& tmpDrawingPriority = Owner().GetDrawPriority ();
-
-   // Expand coastlines request
-   MvRequest coastRequest = viewRequest_.getSubrequest ("COASTLINES");
-   MvRequest expRequest = ObjectList::ExpandRequest (coastRequest, EXPAND_DEFAULTS);
-
-   // Account for land-sea shading and for the drawing order
-   // Check if Coastline Land Shade is to be drawn on the background
-   const char* onoff = (const char*)expRequest ( "MAP_COASTLINE_LAND_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE_LAND_SHADE" );
-      if ( drawPrior == 999 )
-         coastRequest ("MAP_COASTLINE_LAND_SHADE") = "OFF";
-      else
-         draw = true;
-   }
-
-   // Check if Coastline Sea Shade is to be drawn on the background
-   onoff = (const char*)expRequest ( "MAP_COASTLINE_SEA_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE_SEA_SHADE" );
-      if ( drawPrior == 999 )
-         coastRequest ("MAP_COASTLINE_SEA_SHADE") = "OFF";
-      else
-         draw = true;
-   }
-
-   // Check if Coastline is to be drawn on the background
-   onoff = (const char*)expRequest ( "MAP_COASTLINE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE" );
-      if ( drawPrior == 999 && draw == false )
-         coastRequest ("MAP_COASTLINE") = "OFF";
-      else
-         draw = true;
-   }
-
-   // Draw Coastline on the background
-   if ( draw )
-   {
-      // By default Boundaries, Cities and Rivers are plotted in the Foreground.
-      coastRequest ("MAP_BOUNDARIES") = "OFF";
-      coastRequest ("MAP_CITIES") = "OFF";
-      coastRequest ("MAP_RIVERS") = "OFF";
-
-      // Unset other drawings
-      coastRequest ("MAP_GRID") = "OFF";
-      coastRequest ("MAP_LABEL") = "OFF";
-
-      // Draw the layer info
-      MvIcon icon(coastRequest,true);
-      Owner().DrawLayerInfo( icon.Id() );
-
-      // Ask the graphics engine to draw the coastlines
-      GraphicsEngine& ge = Owner().GetGraphicsEngine();
-      ge.Draw ( coastRequest, true );
-   }
-
-   Owner().SetProjection ( viewRequest_ );
-#endif
 }
 
 void
 MapView::DrawForeground ( )
 {
-#if 1  // New code
    // Retrieve foreground request
    MvRequest foreList;
    if ( this->RetrieveForeground(foreList) == 0 )
@@ -546,99 +447,14 @@ MapView::DrawForeground ( )
    {
       // Draw the layer info
       MvRequest req = foreList.justOneRequest();
-      MvIcon icon(req,true);
-      Owner().DrawLayerInfo( icon.Id() );
+      string icon_name = (const char*)req("_NAME") ? mbasename((const char*)req("_NAME")) : "<coastlines>";
+      Owner().DrawLayerInfo( (int)req("_ID"), icon_name.c_str() );
 
       // Ask the graphics engine to draw the coastlines
       ge.Draw ( req, true );
    
       foreList.advance();
    }
-
-   Owner().SetProjection ( viewRequest_ );  //????
-
-#else  // Old code, delete it later
-   bool draw   = false;
-   string ON   = "ON";
-   string LAST = "LAST";
-
-   // Retrieve the graphics Engine
-   GraphicsEngine& ge = Owner().GetGraphicsEngine();
-
-   // Get DrawingPriority from Page
-   DrawingPriority& tmpDrawingPriority = Owner().GetDrawPriority ();
-
-   // Expand coastlines request
-   MvRequest coastRequest = viewRequest_.getSubrequest ("COASTLINES");
-   MvRequest expRequest = ObjectList::ExpandRequest (coastRequest, EXPAND_DEFAULTS);
-
-   // Account for land-sea shading and for the drawing order
-   // Check if Coastline Land Shade is to be drawn on the foreground
-   const char* onoff = (const char*)expRequest ( "MAP_COASTLINE_LAND_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE_LAND_SHADE" );
-      if ( drawPrior == 999 )
-         draw = true;
-      else
-         coastRequest ("MAP_COASTLINE_LAND_SHADE") = "OFF";
-   }
-
-   // Check if Coastline Sea Shade is to be drawn on the foreground
-   onoff = (const char*)expRequest ( "MAP_COASTLINE_SEA_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE_SEA_SHADE" );
-      if ( drawPrior == 999 )
-         draw = true;
-      else
-         coastRequest ("MAP_COASTLINE_SEA_SHADE") = "OFF";
-   }
-
-   // Check if Coastline is to be drawn on the foreground
-   onoff = (const char*)expRequest ( "MAP_COASTLINE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-   {
-      int drawPrior = tmpDrawingPriority.GetPriority ( "COASTLINE" );
-
-      // If MAP_COASTLINE was already ploted in the background but SEA/LAND
-      // SHADE was selected, MAGICS needs that MAP_COASTLINE be set to ON.
-      // If both MAP_COASTLINE and SEA/LAND SHADE are to be drawn,
-      // Magics needs a double PCOAST
-      if ( drawPrior != 999 && draw == false )
-         coastRequest ("MAP_COASTLINE") = "OFF";
-      else if ( drawPrior == 999 && draw == true )
-      {
-         // Unset other drawings and set land/sea shade request
-         MvRequest finalRequest = coastRequest;
-         finalRequest ("MAP_GRID")  = "OFF";
-         finalRequest ("MAP_LABEL") = "OFF";
-         finalRequest ("MAP_BOUNDARIES") = "OFF";
-         finalRequest ("MAP_CITIES") = "OFF";
-         finalRequest ("MAP_RIVERS") = "OFF";
-
-         // Unset Sea/Land Shade
-         coastRequest ("MAP_COASTLINE_LAND_SHADE") = "OFF";
-         coastRequest ("MAP_COASTLINE_SEA_SHADE")  = "OFF";
-
-         // Draw the layer info
-         MvIcon icon(finalRequest,true);
-         Owner().DrawLayerInfo( icon.Id() );
-
-         // Draw Coastline sea/land shade
-         ge.Draw ( finalRequest, true );
-      }
-   }
-
-   // Draw the layer info
-   MvIcon icon(coastRequest,true);
-   Owner().DrawLayerInfo( icon.Id() );
-
-   // Draw Coastline and/or other options (Grid,Label,...) on the foreground
-   ge.Draw ( coastRequest, true );
-
-   Owner().SetProjection ( viewRequest_ );
-#endif
 }
 
 // Describe the contents of the view
@@ -701,68 +517,75 @@ bool MapView::CallService(const MvRequest &req, PmContext &context)
 
 int MapView::RetrieveBackground( MvRequest& backReq )
 {
-   // Get icons from the Database first. If there is none then get the icon
-   // from the View.
-   int ncount = this->RetrieveBackForeground("BACKGROUND",backReq);
-   if ( ncount )
-       return ncount;
+   // Get icons from the Database first.
+   // If there is none then get icons from the View (embedded icon).
+   // If there is none then create a default Coastlines.
+   int nicons = this->RetrieveBackForeground("BACKGROUND",backReq);
+   if ( nicons )
+      return nicons;   // icons retrieved from the Database
 
    // Retrieve icon from the View
-   // By default, Coastline will only be plotted on the foreground, unless
-   // there is a sea/land shade to be drawn on the background
-   backReq = viewRequest_.getSubrequest ("COASTLINES");
-
-   // Account for land-sea shading defined in the View.
-   // By default, if Coastline Land Shade is ON it will be drawn on the background
-   bool draw = false;
-   const char* onoff = (const char*)backReq( "MAP_COASTLINE_LAND_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-      draw = true;
-
-   // By default, if Coastline Sea Shade is ON it will be drawn on the background
-   onoff = (const char*)backReq( "MAP_COASTLINE_SEA_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-      draw = true;
-
-   // Set Coastline to be drawn on the background
-   if ( draw )
+   bool iconDefault = false;
+   MvRequest req = viewRequest_.getSubrequest ("COASTLINES");
+   if ( !req )
    {
-      // By default Boundaries, Cities and Rivers are plotted on the Foreground
-      backReq ("MAP_BOUNDARIES") = "OFF";
-      backReq ("MAP_CITIES") = "OFF";
-      backReq ("MAP_RIVERS") = "OFF";
+      // Create a default Coastlines
+      req = ObjectList::CreateDefaultRequest (MCOAST);
+      iconDefault = true;
+   }
 
-      // Unset other drawings
-      backReq ("MAP_GRID") = "OFF";
-      backReq ("MAP_LABEL") = "OFF";
+   // By default, Coastline will only be plotted on the foreground, unless
+   // there is a sea/land shade to be drawn on the background.
+   Owner().CoastlinesBackground(req,backReq);
+
+   // Create an icon with an ID
+   if ( backReq )
+   {
+      MvIcon icon(backReq,true);
+      backReq = icon.Request();
+
+      // Add icon to the DataBase
+      int id = (iconDefault) ? Owner().FindSuperPage()->Id() : Owner().Id();
+      MvIconDataBase& dataBase = Owner().IconDataBase();
+      dataBase.InsertIcon( PRES_VISDEF_REL, id, icon );
 
       return 1;
    }
 
-   backReq.clean();
    return 0;
 }
 
 int MapView::RetrieveForeground( MvRequest& foreReq )
 {
-   // Get icons from the Database first. If there is none then get the icon
-   // from the View.
-   int ncount = this->RetrieveBackForeground("FOREGROUND",foreReq);
-   if ( ncount )
-      return ncount;
+   // Get icons from the Database first.
+   // If there is none then get icons from the View (embedded icon).
+   // If there is none then create a default Coastlines.
+   int nicons = this->RetrieveBackForeground("FOREGROUND",foreReq);
+   if ( nicons )
+      return nicons;   // icons retrieved from the Database
 
    // Retrieve icon from the View
-   foreReq = viewRequest_.getSubrequest ("COASTLINES");
+   bool iconDefault = false;
+   MvRequest req = viewRequest_.getSubrequest ("COASTLINES");
+   if ( !req )
+   {
+      // Create a default Coastlines
+      req = ObjectList::CreateDefaultRequest (MCOAST);
+      iconDefault = true;
+   }
 
    // Account for land-sea shading. If they are setted to ON then 
-   // they were previously drawn on the Background by default. 
-   const char* onoff = (const char*)foreReq ( "MAP_COASTLINE_LAND_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-      foreReq ("MAP_COASTLINE_LAND_SHADE") = "OFF";
+   // they were previously drawn on the Background by default.
+   Owner().CoastlinesForeground(req,foreReq);
 
-   onoff = (const char*)foreReq ( "MAP_COASTLINE_SEA_SHADE" );
-   if ( onoff && strcmp(onoff,"ON") == 0 )
-      foreReq ("MAP_COASTLINE_SEA_SHADE") = "OFF";
+   // Create an icon with an ID
+   MvIcon icon(foreReq,true);
+   foreReq = icon.Request();
+
+   // Add icon to the DataBase
+   int id = iconDefault ? Owner().FindSuperPage()->Id() : Owner().Id();
+   MvIconDataBase& dataBase = Owner().IconDataBase();
+   dataBase.InsertIcon( PRES_VISDEF_REL, id, icon );
 
    return 1;
 }
@@ -771,9 +594,12 @@ int MapView::RetrieveBackForeground( const char* stype, MvRequest& req )
 {
    // Get icons from the database (if any)
    req.clean();
-   MvIconDataBase& dataBase = Owner().IconDataBase();
    MvIconList iconList;
-   if ( dataBase.RetrieveIcon ( PRES_VISDEF_REL, Owner().Id(), iconList ) == 0 )
+   bool usingDefault = Owner().DefaultVisDefList ( MCOAST, iconList, 0, GETBYVISDEF);
+   if ( usingDefault )
+      usingDefault = Owner().DefaultVisDefList ( PCOAST, iconList, 0, GETBYVISDEF);
+
+   if ( usingDefault )
       return 0;
 
    // Save the requested type icons
@@ -782,7 +608,7 @@ int MapView::RetrieveBackForeground( const char* stype, MvRequest& req )
    for ( ii = iconList.begin(); ii != iconList.end(); ii++ )
    {
       MvRequest currentReq = (*ii).Request();
-      if ( ObjectList::IsVisDefBackForeground(currentReq.getVerb()) )
+      if ( ObjectList::IsVisDefCoastlines(currentReq.getVerb()) )
       {
          if ( (const char*)currentReq("_PLOTTING_ORDER") &&
               (string(stype) == (const char*)currentReq("_PLOTTING_ORDER")) )
